// Interface to mastodon (etc) instances

use megalodon::entities::UploadMedia;
use megalodon::generator;
use megalodon::megalodon::PostStatusInputOptions;
use megalodon::{self, megalodon::UploadMediaInputOptions};
use std::io::Cursor;

use crate::{StdError, APP_USER_AGENT};

const FEDI_ACCESS_TOKEN_ENV_VAR: &str = "FEDI_ACCESS_TOKEN";
const FEDI_INSTANCE_ENV_VAR: &str = "FEDI_INSTANCE";

pub async fn toot(text: &str, img: Option<Box<Cursor<Vec<u8>>>>) -> StdError<()> {
    let client = megalodon::generator(
        megalodon::SNS::Pleroma,
        std::env::var(FEDI_INSTANCE_ENV_VAR)
            .unwrap_or_else(|_| panic!("{} env var not present", FEDI_ACCESS_TOKEN_ENV_VAR))
            .into(),
        Some(
            std::env::var(FEDI_ACCESS_TOKEN_ENV_VAR)
                .unwrap_or_else(|_| panic!("{} env var not present", FEDI_ACCESS_TOKEN_ENV_VAR))
                .into(),
        ),
        Some(APP_USER_AGENT.into()),
    );
    let mut ops = PostStatusInputOptions::default();
    if let Some(img) = img {
        let media = client
            .upload_media_reader(img, Some(&UploadMediaInputOptions::default()))
            .await?;
        ops.media_ids = Some(vec![match media.json {
            UploadMedia::Attachment(a) => a.id,
            UploadMedia::AsyncAttachment(a) => a.id,
        }]);
    }
    client.post_status(text.into(), Some(&ops)).await?;
    Ok(())
}

pub async fn authorize_fedi() -> StdError<()> {
    let url = std::env::var(FEDI_INSTANCE_ENV_VAR)
        .unwrap_or_else(|_| panic!("{} env var not present", FEDI_ACCESS_TOKEN_ENV_VAR))
        .into();
    let client = generator(megalodon::SNS::Pleroma, url, None, None);
    let options = megalodon::megalodon::AppInputOptions {
        scopes: Some([String::from("read"), String::from("write")].to_vec()),
        ..Default::default()
    };

    match client.register_app(String::from("iso7010"), &options).await {
        Ok(app_data) => {
            println!("{}", app_data.url.unwrap());
            println!("Enter code:");
            let mut code = String::new();
            std::io::stdin().read_line(&mut code).ok();
            match client
                .fetch_access_token(
                    app_data.client_id,
                    app_data.client_secret,
                    code.trim().to_string(),
                    megalodon::default::NO_REDIRECT.to_string(),
                )
                .await
            {
                Ok(token_data) => {
                    println!("token: {}", token_data.access_token);
                    if let Some(refresh) = token_data.refresh_token {
                        println!("refresh_token: {}", refresh);
                    }
                }
                Err(err) => {
                    panic!("{}", err);
                }
            }
        }
        Err(err) => {
            panic!("{}", err);
        }
    }
    Ok(())
}
